/*  OTRASH.C                                                */
/* "Trash-Can" File Delete utility for Windows 3.1          */
/* Inspired by original TRASH.C which is:
       Copyright (C) 1992, John D. Ruley, Windows Magazine 
/* This version contains very little of the original's code.
/* In the spirit of the original:
   This program is absolutely free, and you may copy and use
   it at will, provided you do not charge for it.
   All other Rights Reserved

/*
  Added features by David L. Jones (BIX:davejones)
  - trash can bulges when files are dumped in it.
  - delete is delayed: files in the can are hidden.
    Files marked as hidden when put in the trash are handled OK.
  - files can be untrashed by opening the window and 
    double clicking their entries.
  - The trashcan empties on command
  - Files can be left in the can between sessions.  They
    are detected on restart (unless someone clobbers OTRASH.INI).
  - Guest appearance by a famous trashophile.
*/

#include <windows.h>
#include <stdio.h>
#include <dos.h>
#include <errno.h>
#include <string.h>
#include "trash.h"
#include "resid.h"

#define WINDOWCAPTION "O'Trash"
#define TRASHLIST     "TrashedFiles"
#define TRASHINI      "OTRASH.INI"

int PASCAL WinMain(HANDLE hInstance,
                   HANDLE hPrevInstance,
                   LPSTR  lpszCmdLine,
                   int    nCmdShow)
{
 /***********************************************************************/
 /* HANDLE hInstance;       handle for this instance                    */
 /* HANDLE hPrevInstance;   handle for possible previous instances      */
 /* LPSTR  lpszCmdLine;     long pointer to exec command line           */
 /* int    nCmdShow;        Show code for main window display           */
 /***********************************************************************/

MSG  msg;           /* MSG structure to store your messages*/
int  nRc;           /* return value from Register Classes  */

char szAppName[] = "TRASH";
HWND hWndMain;


    if(!hPrevInstance) {
    /* register window classes if first instance of application         */
        WNDCLASS   wndclass;    /* struct to define a window class             */

     /* load WNDCLASS with window's characteristics                         */
        wndclass.style          = CS_HREDRAW | CS_VREDRAW | CS_BYTEALIGNWINDOW;
        wndclass.lpfnWndProc    = WndProc;
     /* Extra storage for Class and Window objects                          */
        wndclass.cbClsExtra     = 0;
        wndclass.cbWndExtra     = 0;
        wndclass.hInstance      = hInstance;
        wndclass.hIcon          = NULL;
        wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
     /* Create brush for erasing background                                 */
        wndclass.hbrBackground  = (HBRUSH)(COLOR_BACKGROUND+1);
        wndclass.lpszMenuName   = "TRASHMENU";
        wndclass.lpszClassName  = szAppName;   /* Class Name is App Name */

        if(!RegisterClass(&wndclass)) {
            char szString[80];
       /* registering one of the windows failed                         */
            LoadString(hInstance,
                       IDS_ERR_REGISTER_CLASS,
                       szString,
                       sizeof(szString));

            MessageBox(NULL,
                       szString,
                       NULL,
                       MB_OK | MB_ICONEXCLAMATION);
        }
    }


 /* create application's Main window                                    */
    hWndMain = CreateWindow(
                szAppName,               /* Window class name           */
                WINDOWCAPTION,
                WS_SYSMENU      |        /* Add system menu box         */
                WS_ICONIC       |
                WS_OVERLAPPEDWINDOW,
                CW_USEDEFAULT, 0,        /* Use default X, Y            */
                CW_USEDEFAULT, 0,        /* Use default X, Y            */
                NULL,                    /* Parent window's handle      */
                NULL,                    /* Default to Class Menu       */
                hInstance,               /* Instance of window          */
                NULL);                   /* Create struct for WM_CREATE */


    if(hWndMain == NULL) {
        char szString[80];
        LoadString(hInstance,
                   IDS_ERR_CREATE_WINDOW,
                   szString,
                   sizeof(szString));

        MessageBox(NULL,
                   szString,
                   NULL,
                   MB_OK | MB_ICONEXCLAMATION);

        return IDS_ERR_CREATE_WINDOW;
    }

    ShowWindow(hWndMain, SW_SHOWMINNOACTIVE); /* display main window      */

    DragAcceptFiles(hWndMain, TRUE);

    while(GetMessage(&msg, NULL, 0, 0)) {  /* Until WM_QUIT message    */

        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

} /*  End of WinMain                                                    */

void HandleAddedItems (HWND hWnd, HMENU hSystemMenu)
/* Check the trash can and Enable the system menu items related to it */
{
    int nstring = SendMessage(GetWindow(hWnd, GW_CHILD),
                              LB_GETCOUNT,
                              0,
                              0L
                              );
    EnableMenuItem(hSystemMenu,
                  MAKESYSMENUITEM(IDM_EMPTYTRASH),
                  (nstring > 0 ? MF_ENABLED : MF_GRAYED)
                  );

}
void ResetSystemMenu(HWND hWnd)
/* Restore the default System Menu */
{
    GetSystemMenu(hWnd, TRUE);
}

void SetupSystemMenu(HWND hWnd)
/* Add extra items to the system menu */
{
    HANDLE hSystemMenu = GetSystemMenu(hWnd, TRUE);
    HANDLE hMyMenu     = GetMenu(hWnd);
    char   szBuffer[50];

    if (!hSystemMenu) {/* NULL if previously unmodified */
        hSystemMenu = GetSystemMenu(hWnd, FALSE);
    }

    GetMenuString(hMyMenu,
                  IDM_EMPTYTRASH,
                  szBuffer,
                  sizeof(szBuffer),
                  MF_BYCOMMAND);


    AppendMenu(hSystemMenu,
               (MF_STRING ),
               MAKESYSMENUITEM(IDM_EMPTYTRASH),
               szBuffer);

    HandleAddedItems(hWnd, hSystemMenu);
}

BOOL ProcessListWindow(HWND hWnd, WORD wParam, LONG lParam)
/* Handle events in the List window: just respond to double
   clicks to return files to "life" (actually un-hide them,
   unless they were already hidden, in which case just forget
   them
   */
{
    char        szBuffer[128];
    int sid, nstring;


    if (wParam != IDI_STACK)
        return FALSE;

    if (HIWORD(lParam) == LBN_DBLCLK) {
        nstring = SendMessage(LOWORD(lParam),
                    LB_GETCOUNT,
                    0,
                    0L
                    );

        sid = SendMessage(LOWORD(lParam),
                    LB_GETCURSEL,
                    0,
                    0L
                    );
        SendMessage(LOWORD(lParam),
                    LB_GETTEXT,
                    sid,
                    (LONG) (LPSTR) szBuffer
                    );
        SendMessage(LOWORD(lParam),
                    LB_DELETESTRING,
                    sid,
                    0L
                    );
                    /* Delete the entry in the INI file */
        WritePrivateProfileString((LPSTR)TRASHLIST,
                                  (LPSTR)szBuffer,
                                  NULL,
                                  (LPSTR)TRASHINI);

        if (sid == nstring - 1 ) /* special if we deleted the last string */
            sid = sid - 1;
        /* Cause the hilite to appear in the right place on the list (cosmetic) */
        if (nstring > 1) {
            SendMessage(LOWORD(lParam),
                 LB_SETCURSEL,
                 sid,
                 0L
                 );
        }

        if (_fstrlen(szBuffer) && (szBuffer[1] != '!')) {
        /* un-hide any previously un-hidden file */
            unsigned attrib;
            _dos_getfileattr(szBuffer, &attrib);
            _dos_setfileattr(szBuffer, attrib &  (~_A_HIDDEN));
        }
    }
    return TRUE;
}

SizeChildWindow(HWND   hWnd,
                WORD   wParam,
                LONG   lParam)
/* Make sure size of list box tracks with the parent window */
{
    int cxClient = LOWORD(lParam);
    int cyClient = HIWORD(lParam);

    MoveWindow(GetWindow(hWnd, GW_CHILD),
           0,
           0,
           cxClient,
           cyClient,
           TRUE
           );

}

DrawOscar(HWND hWnd) {
/* Execute picture of a friend of ours */
    HANDLE hInst = GetWindowWord(hWnd, GWW_HINSTANCE);

    HICON hOscar = LoadIcon(hInst, "TRASHOSCAR");
    HDC hDC      = GetWindowDC(hWnd);
    SendMessage(hWnd, WM_ERASEBKGND, (WORD)hDC, 0L);
    DrawIcon(hDC, 0,0, hOscar);
    ReleaseDC(hWnd, hDC);
}
int DeleteFile(HWND hWnd, PSTR szFile)
/* Delete a file with user intervention/notification as necessary */
{
    BOOL bRetry     = TRUE;
    BOOL bContinue  = TRUE;
    int  nAns       = IDYES;
    char s[80];
    unsigned err, attrib;

    for (;bRetry;) {
        err = remove(szFile);
        bRetry = FALSE;

        if (err == -1) {
            if (errno == EACCES) {
                _dos_getfileattr(szFile, &attrib);
                if (attrib & _A_RDONLY) {
                    wsprintf((LPSTR)s,
                            (LPSTR)"Delete Readonly file: %s ?",
                            (LPSTR)(szFile));
                    nAns = MessageBox(  hWnd,
                                        s,
                                        WINDOWCAPTION,
                                        MB_YESNOCANCEL | MB_ICONQUESTION);
                    if (nAns == IDYES) {
                        _dos_setfileattr(szFile, attrib &  (~_A_RDONLY));
                        bRetry = TRUE;
                    }
                }
                else {
                    wsprintf(s,
                            (LPSTR)"Unable to delete %s -- Access Denied!",
                            (LPSTR)(szFile));
                    nAns = MessageBox(  hWnd,
                                        s,
                                        WINDOWCAPTION,
                                        MB_OKCANCEL | MB_ICONEXCLAMATION);
                }
            }
            else {
                if(errno == ENOENT) {
                    wsprintf(s,
                            (LPSTR)"Unable to delete %s -- No such path!",
                            (LPSTR)szFile);
                }
                else {
                    wsprintf(s,
                            "Unable to delete %s -- Reason Unknown!",
                            (LPSTR)szFile);
                }
                nAns = MessageBox( hWnd,
                                   s,
                                   WINDOWCAPTION,
                                   MB_OKCANCEL | MB_ICONEXCLAMATION);
            }
        }
    }
    return nAns;
}
BOOL EmptyTrash(HWND hWnd)
/* Delete all the files in the list window */
/* Since the strings in the list move up as we delete
/* them, we do some fancy pointer footwork to account
/* for cases where we skip a delete
/**/
{
    int        nContinue = IDYES;
    char       szBuffer[80];
    int sid, nstring, nchar, nList;

    HWND  hList = GetWindow(hWnd, GW_CHILD);

    nstring = SendMessage(hList,
                    LB_GETCOUNT,
                    0,
                    0L
                    );

    if (nstring && IsIconic(hWnd))
        DrawOscar(hWnd);

    nList = 0; /* pointer to active list item: usually the top one */
    for (sid = 0, nContinue = IDOK;
        (sid < nstring) && (nContinue != IDCANCEL)
         ; sid++) {

        SendMessage(hList,
                    LB_GETTEXT,
                    nList,
                    (LONG)(LPSTR)szBuffer);
        /* files that are really hidden are marked with a "!" */
        nchar = (szBuffer[0] == '!')? 1 : 0 ;

        SetWindowText(hWnd, (LPSTR)(szBuffer+nchar));

        nContinue = DeleteFile(hWnd, szBuffer+nchar);

        if (nContinue == IDYES) {
            WritePrivateProfileString((LPSTR)TRASHLIST,
                                      (LPSTR)szBuffer,
                                      NULL,
                                      (LPSTR)TRASHINI);
            SendMessage(hList,
                        LB_DELETESTRING,
                        nList,
                        0L
                        );
        }
        else {
            nList++; /* move the pointer past this item */
        }
    }

    SetWindowText(hWnd, (LPSTR)WINDOWCAPTION);

    if (nstring && IsIconic(hWnd)) {
        InvalidateRect(hWnd, NULL, TRUE);
    }
    return TRUE;
}

ProcessMenuCommand(HWND hWnd, WORD wMsgID, WORD wParam, LONG lParam)
/* DO menu actions, or enable the item(s).... */
{
    if (wMsgID == WM_INITMENU) {
        int nstring = SendMessage(GetWindow(hWnd, GW_CHILD),
                                  LB_GETCOUNT,
                                  0,
                                  0L
                                  );
        EnableMenuItem(wParam,
                      IDM_EMPTYTRASH,
                      (nstring > 0 ? MF_ENABLED : MF_GRAYED)
                      );

    }
    else {
        switch (wParam) {
            case IDM_EMPTYTRASH:
                EmptyTrash(hWnd);
                break;
            case IDM_EXIT:
                PostMessage(hWnd, WM_CLOSE, 0, 0L);
                break;
            default:
                break;
        }
    }

}

CheckTrash(HWND hWnd)
/* Ask to delete trash if any found: used on shutdown & end of Windows session */
{
    if (SendMessage(GetWindow(hWnd, GW_CHILD), LB_GETCOUNT, 0, 0L)) {
        if (IDYES == MessageBox(hWnd,
                                "Empty Trash?",
                                WINDOWCAPTION,
                                MB_YESNO | MB_ICONQUESTION ))
            EmptyTrash(hWnd);
    }
}

/* Grab any files listed in the INI file and put them in the list window:
   used during startup
*/
FillListWindow(HWND hList)
{
    HANDLE hMem   = GlobalAlloc(GHND, 20000);
    LPSTR  lpList = GlobalLock(hMem);
    int    nchars;
    char   szFile[128];

    if (lpList) {
        /* Get all the keys in the trashlist: this could be a long list ! */
        nchars = GetPrivateProfileString((LPSTR)TRASHLIST,
                                        NULL,
                                        (LPSTR)"",
                                        (LPSTR)lpList,
                                        20000,
                                        (LPSTR)TRASHINI
                                        );
                                        
                                        
        /* Parse the trashlist: its a lot of null-terminated strings, terminated
           itself by a double-null
           */                                        

        if (nchars) {
            int m = 0;
            int n = 0;
            for (; (m < nchars) && (n >= 0);) {
                if (lpList[m]) {
                    szFile[n++] = lpList[m];
                }
                else {
                    if (n) {
                        szFile[n] = '\0';
                        SendMessage(hList,
                                    LB_ADDSTRING,
                                    0,
                                    (LONG)(LPSTR)szFile);
                        n = 0;
                    }
                    else {
                        n = -1;
                    }
                }
                m++;
            }
        }
        GlobalUnlock(hMem);
    }
    if (hMem) {
        GlobalFree(hMem);
    }
}


/************************************************************************/
/*                                                                      */
/* Main Window Procedure                                                */
/*                                                                      */
/* This procedure provides service routines for the Windows events      */
/* (messages) that Windows sends to the window, as well as the user     */
/* initiated events (messages) that are generated when the user selects */
/* the action bar and pulldown menu controls or the corresponding       */
/* keyboard accelerators.                                               */
/*                                                                      */
/************************************************************************/

LONG FAR PASCAL WndProc(HWND hWnd, WORD wMessage, WORD wParam, LONG lParam)
{
 HMENU       hMenu=0;            /* handle for the menu                 */
 HWND        hList;              /* handle for list of files to delete  */
 HDC         hDC;                /* handle for the display device       */
 PAINTSTRUCT ps;                 /* holds PAINT information             */

 switch (wMessage)
   {


    case WM_CREATE:
        hList = CreateWindow(
                              "listbox",
                              NULL,
                              CHILDSTYLE
                              | WS_VSCROLL
                              | LBS_NOINTEGRALHEIGHT
                              | LBS_STANDARD,
                              0,
                              0,
                              0,
                              0,
                              hWnd,
                              IDI_STACK,
                              ((LPCREATESTRUCT)lParam)->hInstance,
                              NULL
                              );
        if (hList) {
            FillListWindow(hList);
            SetupSystemMenu(hWnd);
        }
        else {
            return FALSE;
        }

        break;       /*  End of WM_CREATE                              */


    case WM_SYSCOMMAND: /* a system menu item has been picked */
        {
            if (wParam > 0xF000) {   /* a standard item */
                return DefWindowProc(hWnd, wMessage, wParam, lParam);
            }
            else {    /* an item added by me */
                ProcessMenuCommand(hWnd,
                                    WM_COMMAND,
                                    SYSTOMYMENU(wParam),
                                    0L);
            }
        }
        break;

    case WM_INITMENU:
        if (wParam == GetMenu(hWnd)) { /* for my menu just pass on thru */
            ProcessMenuCommand(hWnd, wMessage, wParam, lParam);
        }
        else { /* system menu */
            HandleAddedItems(hWnd, wParam); /* check and enable any additions */
            return DefWindowProc(hWnd, wMessage, wParam, lParam);
        }
        break;

    case WM_SIZE:     /*  code for sizing client area                   */
        SizeChildWindow(hWnd, wParam, lParam);
        break;       /* End of WM_SIZE                                 */

    case WM_PAINT:    /* code for the window's client area              */
        hDC = BeginPaint(hWnd, &ps);
        if (IsIconic(hWnd)) {
            HICON hIcon;
            DWORD nch = SendMessage(GetWindow(hWnd, GW_CHILD),
                              LB_GETCOUNT,
                              0,
                              0L
                              );

            if (nch > 0) {
                hIcon = LoadIcon(GetWindowWord(hWnd, GWW_HINSTANCE),
                                  "TRASHFULL");
            }
            else {
                hIcon = LoadIcon(GetWindowWord(hWnd, GWW_HINSTANCE),
                                  "TRASH");
            }
            DrawIcon(hDC, 0, 0, hIcon);
        }

        EndPaint(hWnd, &ps);
        break;       /*  End of WM_PAINT                               */

    case WM_COMMAND:
        if (!ProcessListWindow(hWnd, wParam, lParam))
             ProcessMenuCommand(hWnd, wMessage, wParam, lParam);
        break;

    case WM_CLOSE:  /* close the window                                 */
         CheckTrash(hWnd);
         /* Destroy child windows, modeless dialogs, then, this window  */
         DestroyWindow(hWnd);
         PostQuitMessage(0);  /* Quit the application                 */
         break;

    case WM_QUERYENDSESSION:
         CheckTrash(hWnd);
         return DefWindowProc(hWnd, wMessage, wParam, lParam);
         break;

    case WM_DROPFILES:
        {
            char szBuffer[128];
            HWND  hList         = GetWindow(hWnd, GW_CHILD);
            DWORD nch           = SendMessage(hList, LB_GETCOUNT,0,0L);
            HANDLE hdrop        = (HANDLE)wParam;
            int    nFiles       = DragQueryFile(hdrop, -1, NULL, 0);
            int    m;

            for(m=0;m<nFiles;m++) {
                unsigned attrib;

                DragQueryFile(hdrop,
                             m,
                           (LPSTR)szBuffer,
                           sizeof(szBuffer)
                           );
                if (_fstrlen(szBuffer)) {
                    char szTemp[128]    = "";

                    _dos_getfileattr(szBuffer, &attrib);
                    if (attrib & _A_HIDDEN) {
                     /* for files that are already hidden, prefix with "!" */
                       _fstrcpy(szTemp,"!");
                    }
                    else {
                        _dos_setfileattr(szBuffer, attrib | _A_HIDDEN);
                    }

                    _fstrcat(szTemp,szBuffer);

                    SendMessage(hList,
                                LB_ADDSTRING,
                                0,
                                (LONG)(LPSTR)szTemp);

                    WritePrivateProfileString((LPSTR)TRASHLIST,
                                              (LPSTR)szTemp,
                                              (LPSTR)"",
                                              (LPSTR)TRASHINI);
                }

            }
            DragFinish(hdrop);
            if ( (nch == 0) && (nFiles > 0) && IsIconic(hWnd)) {
                InvalidateRect(hWnd, NULL, TRUE);
            }
        }
        break;

    default:
         /* For any message for which you don't specifically provide a  */
         /* service routine, you should return the message to Windows   */
         /* for default message processing.                             */

         return DefWindowProc(hWnd, wMessage, wParam, lParam);
   }
 return 0L;
}     /* End of WndProc                                         */
